@@ -133,11 +133,11 @@ pango_cairo_fc_font_get_font_face (PangoCairoFont *font)
     {
       cffont->font_face = cairo_ft_font_face_create_for_pattern (fcfont->font_pattern);
       
-      /* Failure of the above should only occur for out of memory,
-       * we can't proceed at that point
+      /* Unable to create FT2 cairo scaled font.
+       * This means out of memory or a cairo/fontconfig/FreeType bug,
        */
       if (!cffont->font_face)
-	g_error ("Unable to create FT2 cairo font face.\nThis means out of memory or a cairo/fontconfig/FreeType bug");
+        return NULL;
     }
   
   return cffont->font_face;
@@ -154,16 +154,20 @@ pango_cairo_fc_font_get_scaled_font (PangoCairoFont *font)
 
       font_face = pango_cairo_fc_font_get_font_face (font);
 
+      if (!font_face)
+        return NULL;
+
       cffont->scaled_font = cairo_scaled_font_create (font_face,
 						      &cffont->font_matrix,
 						      &cffont->ctm,
 						      cffont->options);
 
-      /* Failure of the above should only occur for out of memory,
-       * we can't proceed at that point
+      /* Unable to create FT2 cairo scaled font.
+       * This means out of memory or a cairo/fontconfig/FreeType bug,
+       * or a missing font...
        */
       if (!cffont->scaled_font)
-	g_error ("Unable to create FT2 cairo scaled font.\nThis means out of memory or a cairo/fontconfig/FreeType bug");
+        return NULL;
     }
   
   return cffont->scaled_font;
@@ -173,7 +177,7 @@ pango_cairo_fc_font_get_scaled_font (PangoCairoFont *font)
  *    Method implementations    *
  ********************************/
 
-static void
+static gboolean
 pango_cairo_fc_font_install (PangoCairoFont *font,
 			     cairo_t        *cr)
 {
@@ -183,6 +187,8 @@ pango_cairo_fc_font_install (PangoCairoFont *font,
 		       pango_cairo_fc_font_get_font_face (font));
   cairo_set_font_matrix (cr, &cffont->font_matrix);
   cairo_set_font_options (cr, cffont->options);
+
+  return TRUE;
 }
 
 static void
@@ -290,6 +296,8 @@ pango_cairo_fc_font_lock_face (PangoFcFont *font)
 {
   PangoCairoFont *cfont = (PangoCairoFont *)font;
   cairo_scaled_font_t *scaled_font = pango_cairo_fc_font_get_scaled_font (cfont);
+  if (G_UNLIKELY (!scaled_font))
+    return NULL;
   
   return cairo_ft_scaled_font_lock_face (scaled_font);
 }
@@ -299,6 +307,8 @@ pango_cairo_fc_font_unlock_face (PangoFcFont *font)
 {
   PangoCairoFont *cfont = (PangoCairoFont *)font;
   cairo_scaled_font_t *scaled_font = pango_cairo_fc_font_get_scaled_font (cfont);
+  if (G_UNLIKELY (!scaled_font))
+    return NULL;
   
   cairo_ft_scaled_font_unlock_face (scaled_font);
 }
@@ -346,19 +356,13 @@ pango_cairo_fc_font_get_glyph (PangoFcFont *font,
   return entry->glyph;
 }
 
-static PangoGlyph
-pango_cairo_fc_font_real_get_unknown_glyph (PangoFcFont *font,
-					    gunichar     wc)
-{
-  return wc | PANGO_CAIRO_UNKNOWN_FLAG;
-}
-
 static void
 pango_cairo_fc_font_glyph_extents_cache_init (PangoCairoFcFont *cffont)
 {
   PangoCairoFont *cfont = (PangoCairoFont *)cffont;
   cairo_scaled_font_t *scaled_font = pango_cairo_fc_font_get_scaled_font (cfont);
   cairo_font_extents_t font_extents;
+
   cairo_scaled_font_extents (scaled_font, &font_extents);
 
   cffont->font_extents.x = 0;
@@ -440,17 +444,19 @@ pango_cairo_fc_font_get_glyph_extents (PangoFont      *font,
   PangoCairoFcFont *cffont = (PangoCairoFcFont *)font;
   GlyphExtentsCacheEntry *entry;
 
+  if (!pango_cairo_fc_font_get_scaled_font ((PangoCairoFont *)cffont))
+    {
+      /* Get generic unknown-glyph extents. */
+      pango_font_get_glyph_extents (NULL, glyph, ink_rect, logical_rect);
+      return;
+    }
+
   /* We need to initialize the cache here, since we use cffont->font_extents
    */
   if (cffont->glyph_extents_cache == NULL)
     pango_cairo_fc_font_glyph_extents_cache_init (cffont);
 
-  if (glyph & PANGO_CAIRO_UNKNOWN_FLAG) 
-    {
-      _pango_cairo_get_glyph_extents_missing((PangoCairoFont *)font, glyph, ink_rect, logical_rect);
-      return;
-    }
-  else if (!glyph)
+  if (glyph == PANGO_GLYPH_EMPTY)
     {
       if (ink_rect)
 	*ink_rect = cffont->font_extents;
@@ -458,6 +464,11 @@ pango_cairo_fc_font_get_glyph_extents (PangoFont      *font,
 	*logical_rect = cffont->font_extents;
       return;
     }
+  else if (glyph & PANGO_GLYPH_UNKNOWN_FLAG) 
+    {
+      _pango_cairo_get_glyph_extents_missing((PangoCairoFont *)font, glyph, ink_rect, logical_rect);
+      return;
+    }
 
   entry = pango_cairo_fc_font_get_glyph_extents_cache_entry (cffont, glyph);
 
@@ -514,7 +525,6 @@ pango_cairo_fc_font_class_init (PangoCairoFcFontClass *class)
   fc_font_class->lock_face = pango_cairo_fc_font_lock_face;
   fc_font_class->unlock_face = pango_cairo_fc_font_unlock_face;
   fc_font_class->get_glyph = pango_cairo_fc_font_get_glyph;
-  fc_font_class->get_unknown_glyph = pango_cairo_fc_font_real_get_unknown_glyph;
   fc_font_class->shutdown = pango_cairo_fc_font_shutdown;
 
 #ifdef PROFILE_GLYPH_CACHE
